'''
Simple script to generate the data needed by the SPG SVM trainer.

Computes the kernel matrix based on the sigmoid kernel.
'''

import sys
import optparse
import scipy as sp

def sigmoid(z,  X, sigma):
    '''Sigmoid function that defines the kernel.

    It receives a vector z and a matrix of X and computes the kernel
    of y against all lines of X.
    '''
    dif = z - X
    norms2 = (dif*dif).sum(axis=1)
    return sp.exp(-(norms2/(2.0*sigma**2)))

def kernelMatrix(kernel, samples, out):
    '''Computes the kernel matrix and saves kernel matrix.
    '''
    n_samples, n_feat_plus_1 = samples.shape
    for i in xrange(n_samples):
        line = samples[i, 0]*samples[:, 0]*\
               kernel(samples[i, 1:],samples[:,1:])
        out.write(line.tostring())
        print >>sys.stderr, 'Line', i, 'of kernel matrix ready.'

def extract(train_data, total_pos, total_neg):
    '''Extract exactly total_pos positive samples and total_neg
    negative samples from top of train_data.
    '''
    pos_samples = train_data[train_data[:, 0] == 1][:total_pos]
    neg_samples = train_data[train_data[:, 0] == -1][:total_neg]
    return sp.vstack((pos_samples, neg_samples))
            
def get_options():
    '''Get the options from the command line.
    '''
    usage='usage: %prog [options] data_file train_output npos nneg sigma'
    parser = optparse.OptionParser(usage)
    parser.add_option('-k', '--keep', dest='keep', 
                      action='store_true', default=False, 
                      help='Keep original training data to define final SVM [default %default]')

    opts, args = parser.parse_args()
    try:
        opts.data_file = args[0]
        opts.train_out = args[1]
        opts.npos = int(args[2])
        opts.nneg = int(args[3])
        opts.sigma = eval(args[4])
    except:
        print args
        parser.print_help()
        sys.exit(1)

    return opts             

def save_for_train(data_file, npos, nneg, sigma, train_out, keep):
    '''Read training and test data from data_file and convert it to
    SVM training. 

    Extract npos and nneg samples from classes 1 and -1 respectively
    and compute the kernel matrix using the sigmoid kernel with given
    sigma. Save the training matrix with the classes for svmma.
    
    If keep is true, also save the original training data to recover
    the trained SVM and use it in a test, for example.
    '''
    # Read data and extract train and test
    print 'Processing', data_file
    print 'Read train data and compute kernel matrix...'
    data = sp.load(data_file)
    train_data = extract(data['train'], npos, nneg)
    
    # Save original trainning data if it is needed for test later
    if keep:
        print 'Saving original training data...'
        sp.savez(train_out + '.original_train', train=train_data)

    # Save data for SVM training using svmma (binary form).
    out = file(train_out, 'wb')
    shape = sp.array((train_data.shape[1] - 1, train_data.shape[0]))
    shape = shape.astype('int32')
    out.write(shape.tostring())
    classes = train_data[:, 0].copy()
    out.write(classes.tostring())


    # Compute the kernel matrix saving line by line
    print 'Computing the kernel matrix...'
    def sigmoidKernel(x, y):
        return sigmoid(x, y, sigma)
    kernelMatrix(sigmoidKernel, train_data, out)
    out.close()
    print '# Features', train_data.shape[1] - 1
    print '# Samples', train_data.shape[0]

if __name__ == '__main__':
    options = get_options()
    save_for_train(options.data_file, options.npos, options.nneg, options.sigma,
                   options.train_out, options.keep)
