/*
 * Module Name: svmprob
 *
 * Description: Interface to use SPG code to solve the SVM training
 *    problem described in SECTION 6.2 of the manuscript
 *
 *    R. Cominetti, W. F. Mascarenhas, and Paulo J. S. Silva. "A
 *    Newton method for the continuous quadratic knapsack problem".
 *
 *
 * OBS: It is important to note that this code was written specially
 * to test the continuous quadratic knapsack algorithms. In
 * particular, to force the SPG method to have a single trajectory the
 * search algorithm is always used in the end of the projection
 * routine (throwing away the projection computed by the other
 * algorithms). You can turn this behavior off by switching the macro
 * SINGLETRAJECTORY to 0 below.
 *
 * Copyright: Paulo J. S. Silva <pjssilva@gmail.com> 2012.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "../../lib/opt_cont_quad_knapsack.h"
#include "../../lib/opt_third_party_methods.h"
#include "blg_interface/blg_interface.h"
#include "svmprob.h"

/* Controls if the result really used in the projection is always
   computed by the same algorithm. */
#define SINGLETRAJECTORY 1

/*
  Enumeration of possible projection methods:
     fix = Variable fixing from Kiwiel
     search = Median based from Kiwiel
     newtoncold = Newton method without warm starts
     newtonwarm = Newton method with warm starts
     secantcold = Secant method without warm starts
     secantwarm = Secant method with warm start
     invalid = invalid (not implemented)
*/
typedef enum {
    fix,
    search,
    newtoncold,
    newtonwarm,
    secantcold,
    secantwarm,
    blgcold,
    blgwarm,
    invalid
} projection_method;

/* Global variables defining the problem. Necessary to conform with
   the interface required by the SPG method. */

/* Variables associated to SVM problem */
static int n_feat;             /* Number of features */
static int n_samples;          /* Number of traning samples */
static double *restrict K;     /* Kernel matrix build from sample data */
static double *restrict Kx;    /* Kernel times a vector */
static double *restrict class; /* Vector with sample's classes, +1 or -1. */
static double C;               /* Upper bound to the multipliers,
                                  defines the weight given to the
                                  objectives. */
static double *last_proj;      /* Result of the last projection */

/* Variables associated to the projection problem and its solution */
static cqk_problem p;                 /* The projection problem.   */
static projection_method proj_method; /* Projection method to use  */


/* Information to recover projection statistics. */
static unsigned repetitions; /* Number of times to repeat the
                                projection in order to get a
                                reasonable accurarty in the projection
                                time. */
unsigned n_projs;            /* Total number of projections */
unsigned n_iters;            /* Total number of iterations perfomed by the
                                projection method in all
                                projections. */
unsigned max_iters = 0;      /* Maximum number of iterations in a
                                single projection */
double proj_time;            /* Total time spent in all projections */

/*
 * Function Name: map_method_name
 *
 * Description: maps a projection method name to its code
 *
 * Input:
 *    char *restrict name: name of the desired method.
 *
 * Return value: the method code.
 *
 */
projection_method map_method_name(char *restrict name) {
    if (!strcmp(name, "fix"))
        return fix;
    else if(!strcmp(name, "search"))
        return search;
    else if(!strcmp(name,"newton"))
        return newtoncold;
    else if(!strcmp(name, "newtonwarm"))
        return newtonwarm;
    else if(!strcmp(name, "secant"))
        return secantcold;
    else if(!strcmp(name, "secantwarm"))
        return secantwarm;
    else if(!strcmp(name, "blg"))
        return blgcold;
    else if(!strcmp(name, "blgwarm"))
        return blgwarm;
    else
        return invalid;
}

/*
 * Function Name: set_C
 *
 * Description: Set the partameter C of the SVM problem: the upper
 *    bound for the multipliers. It also updates the projection
 *    description.
 *
 * Input:
 *    duble aC: the new value for C.
 */
void set_C(double aC) {
    int samp;

    C = aC;
    for (samp = 0; samp < p.n; ++samp) {
        if (class[samp] > 0.0) {
            p.low[samp] = 0.0;
            p.up[samp] = C;
        }
        else {
            p.low[samp] = -C;
            p.up[samp] = 0.0;
        }
    }
}

/*
 * Function Name: read_problem
 *
 * Description: Read the SVM problem description (Kernel matrix and
 *    sample classes), from a file
 *
 * Input:
 *    char *restrict problem_name: name of the file.
 *    double aC: value for the parameter C.
 *    char *restrict proj: name of the projection method.
 *
 *    unsigned rep: number of repections for each projection to get a
 *       good running time estimate.
 *
 * Side effects:
 */
void read_problem(char *restrict problem_name, double aC, char *restrict proj,
                  unsigned rep) {
    /* Set solver options */
    repetitions = rep;
    proj_method = map_method_name(proj);
    if (proj_method == invalid) {
        fprintf(stderr, "Invalid projection method %s...exiting.\n",
                proj);
        exit(1);
    }

    /* Open the input file */
    FILE *input = fopen(problem_name, "rb");
    if (input == NULL) {
        fprintf(stderr, "There is no data file named %s...exiting.\n",
                problem_name);
        exit(1);
    }

    /* Read problem dimensions */
    FSCANF(input, "%4c", 1, (char *)(&n_feat));
    FSCANF(input, "%4c", 1, (char *)(&n_samples));

    /* Allocate space to hold data */
    K = (double *) malloc(n_samples*n_samples*sizeof(double));
    Kx = (double *) malloc(n_samples*sizeof(double));
    class = (double *) malloc(n_samples*sizeof(double));
    last_proj = (double *) malloc(n_samples * sizeof(double));
    if (!K || !Kx || !class || !last_proj) {
        fprintf(stderr, "Memory allocation failed at %s, line %d\n",
                __FILE__, __LINE__);
        exit(1);
    }

    /* Read data classes */
    for (unsigned samp = 0; samp < n_samples; ++samp) {
        double signal;

        FSCANF(input, "%8c", 1, (char *) (&signal));
        class[samp] = signal > 0. ? 1.0 : -1.0;
    }

    /* Read kernel matrix (it is symmetric) */
    for (unsigned line = 0; line < n_samples; ++line) {
        for (unsigned col = 0; col < n_samples; ++col)
            FSCANF(input, "%8c", 1, (char*) (K + line*n_samples + col));
    }
    fclose(input);

    /* Allocate space to hold projection problem and intialize its
       constant data */
    allocate_cqk_problem(n_samples, &p);
    for (unsigned samp = 0; samp < p.n; ++samp) {
        p.d[samp] = 1.0;
        p.b[samp] = 1.0;
    }
    set_C(aC);

    /* Zero initial statistics */
    n_projs = 0;
    n_iters = 0;
    proj_time = 0.0;
}

/*
 * Function Name: free_problem
 *
 * Description: Free memory allocated to hold the SVM problem.
 */
void free_problem() {
    free(K);
    free(Kx);
    free(class);
    free_cqk_problem(&p);
    free(last_proj);
}

/*
 * Function: print_statistics
 *
 * Description: Prints the performance statistics for solving the
 *    problem.
*/
void printStatistics() {
    printf("Number of projections: %d\n", n_projs);
    printf("Mean steps: %f\n", (double) n_iters/n_projs);
    printf("Maximum steps: %d\n", max_iters);
    printf("Mean time: %e\n", proj_time/n_projs/repetitions);
}

/*
 * Function Name: matrix_multiply
 *
 * Description: Naive matrix times vectoron, where the matrix is
 *     stored by row.
 *
 * Input:
 *
 * Input/Output
 *    double *retrict A: the matrix.
 *    int m, int n: number of lines and columns of A.
 *    double *restrict x: vector to multiply by A.
 *
 * Output:
 *     double *restrict y: Ax.
 */

/* If LAPACK is available we need dsymv prototype */
#ifdef LAPACK
int dgemv_(char *trans, int *m, int *n, double *alpha,
           double *a, int *lda, double *x, int *incx,
           double *beta, double *y, int *incy);
int dsymv_(char *uplo, int *n, double *alpha,
           double *a, int *lda, double *x, int *incx,
           double *beta, double *y, int *incy);
#endif


void matrix_multiply(double *restrict A, int m, int n,
                     double *restrict x, double *restrict y) {
#ifdef LAPACK
    double
        alpha = 1.0,
        beta = 0.0;
    int
        inc1 = 1;
    dsymv_("l", &n, &alpha, A, &n, x, &inc1, &beta, y, &inc1);
#else
    for (unsigned i = 0; i < m; ++i) {
        y[i] = 0.0;
        for (unsigned j = 0; j < n; ++j)
            y[i] += A[i*n + j]*x[j];
    }
#endif
}

/* Helper functions to allow projection with and without warm starts */

/*
 * Function Name: warmProj
 *
 * Description: Projection algorithm with warm starts. It only accepts
 *    the Newton and secatn methods as there is no clean way to do
 *    variable fixing or median finding with warm starts.
 *
 * Input:
 *    cqk_problem *restrict p: projection problem.
 *
 * Output:
 *    double *restrict x: the computed projection.
*/
int warmProj(cqk_problem *restrict p, double *restrict x) {
    /* Holds the initial point for the warm start */
    static double *x0 = NULL;
    /* Last step required to find a good multiplier. It is used in the
       warm start of the secant method. */
    static double deltaLambda = -1.0;

    int flag = 0;           /* Holds the status of the projection */
    time_t start = clock(); /* Starting time for the projections */
    for (unsigned i = 0; i < repetitions; ++i) {
        DEBUG_PRINT("\nStarting projection\n");

        /* Use hot starts after first iteration */
        if (n_projs > 0)
            x0 = last_proj;

        switch (proj_method) {
        case secantwarm:
            flag = secant(p, x0, &deltaLambda, x);
            break;
        case newtonwarm:
            flag = newton(p, x0, x);
            break;
        case blgwarm:
            flag = blg_interface(p, x0, x);
            break;
        default:
            flag = -1;
        }
    }
    proj_time += ((double) (clock() - start))/CLOCKS_PER_SEC;
    ++n_projs;

    n_iters += flag;
    /* Keep the largest number of iterations used. */
    if (flag > max_iters)
        max_iters = flag;

    /* Copy last projection to estimate next staring point. */
    for (unsigned i = 0; i < p->n; ++i)
        last_proj[i] = x[i];

    return flag >= 0 ? 0 : 1;
}

/*
 * Function Name: coldProj
 *
 * Description: Projection algorithm without warm starts.
 *
 * Input:
 *    cqk_problem *restrict p: projection problem.
 *
 * Output:
 *    double *restrict x: the computed projection.
*/
int coldProj(cqk_problem *restrict p, double *restrict x) {
    double delta_lambda;    /* Used to call the secant method. */
    int flag = 0;           /* Holds the status of the projection */

    time_t start = clock(); /* Starting time for the projections */
    for (unsigned i = 0; i < repetitions; ++i) {
        DEBUG_PRINT("\nStarting projection\n");

        switch (proj_method) {
        case fix:
            flag = kiwiel_var_fix(p, x);
            break;
        case newtoncold:
            flag = newton(p, NULL, x);
            break;
        case search:
            flag = kiwiel_search(p, x);
            break;
        case secantcold:
            delta_lambda = -1.0;
            flag = secant(p, NULL, &delta_lambda, x);
            break;
        case blgcold:
            flag = blg_interface(p, NULL, x);
            break;
        default:
            flag = -1;
        }
    }
    proj_time += ((double) (clock() - start))/CLOCKS_PER_SEC;
    ++n_projs;

    n_iters += flag;
    /* Keep the largest number of iterations used. */
    if (flag > max_iters)
        max_iters = flag;

    return flag >= 0 ? 0 : 1;
}

/*
 * Function Name: evalf
 *
 * Description: Evaluates the objective functin f(x) = 1/2 x'Kx - e'x.
 *
 * Input:
 *    int n: dimension of x.
 *    double *restrict x: function argument
 *
 * Output:
 *    double *restict f: objective function value
 *    int *restrict flag: Returns 0 to indicate that the objective was
 *       successfully computed.
*/
void evalf(int n, double *restrict x,
           double *restrict f, int *restrict flag) {

    int i;

    /* Compute f */

    /* Compute Kx */
    matrix_multiply(K, n_samples, n_samples, x, Kx);
    *f = 0.0;
    for (i = 0; i < n_samples; ++i) {
        *f += 0.5*x[i]*Kx[i] - x[i];
    }
    *flag = 0;
}

/*
 * Function Name: evalg
 *
 * Description: Compute the gradient of the objective function g(x) =
 *    Kx - e.
 *
 * Input:
 *    int n: dimension of x.
 *    double *restrict x: function argument.
 *
 * Output:
 *    double *restrict g: the gradient
 *    int *restrict flag: Returns 0 to indicate that the objective was
 *       successfully computed.
 *
 * WARNING: This function assumes that evalf was called just before
 *    with the same x.
*/
void evalg(int n, double *restrict x,
           double *restrict g, int *restrict flag) {

    for (unsigned i = 0; i < n_samples; ++i) {
        g[i] = Kx[i] - 1.0;
    }
    *flag = 0;
}

/*
 * Function Name: proj
 *
 * Description: Solve the projection onto 0 <= x <= C, signals'x = 0
 * (the feasible set for the SVMN problem using the correct method.
 *
 * Input:
 *    int n: dimension of x;
 *
 * Input/Outuput: double *restrict x: The point the project at input
 *    and its projection at output.
 *
 *    int *restrict flag: Returns 0 to indicate that the objective was
 *       successfully computed.
 */
void proj(int n, double *restrict x, int *restrict flag) {

    /* Adapt signs of the point to project to the projection code */
    for(unsigned i = 0; i < n; ++i)
        if (class[i] > 0.0)
            p.a[i] = x[i];
        else
            p.a[i] = -x[i];
    p.r = 0.0;

    if (proj_method == newtonwarm || proj_method == secantwarm ||
        proj_method == blgwarm)
        *flag = warmProj(&p, x);
    else
        *flag = coldProj(&p, x);

    /* Always call Newton projection to force SPG to take the same
       trajectory, if requested. */
    if (SINGLETRAJECTORY) {
        *flag = kiwiel_search(&p, x);
        *flag = !(*flag >= 0);
    }

    /* Fix projection signs */
    for (unsigned i = 0; i < n_samples; ++i)
        if (class[i] < 0) x[i] = -x[i];
}
