/*
 * Module Name: svmprob
 *
 * Description: Interface to use SPG code to solve the SVM training
 *    problem described in SECTION 6.2 of the manuscript
 *
 *    R. Cominetti, W. F. Mascarenhas, and Paulo J. S. Silva. "A
 *    Newton method for the continuous quadratic knapsack problem".
 *
 *
 * Copyright: Paulo J. S. Silva <pjssilva@gmail.com> 2012.
 */

#ifndef SVMPROB_H
#define SVMPROB_H

/* Helper macro to test (badly) file read */
#define FSCANF(file_p, format, n_reads, ...) \
    if (fscanf(file_p, format, __VA_ARGS__) < n_reads) {              \
        fprintf(stderr, "Detected bad file format at %s, line %d.\n", \
                __FILE__, __LINE__); \
        exit(1); \
    }

/*
 * Function Name: read_problem
 *
 * Description: Read the SVM problem description (Kernel matrix and
 *    sample classes), from a file
 *
 * Input:
 *    char *restrict problem_name: name of the file.
 *    double aC: value for the parameter C.
 *    char *restrict proj: name of the projection method.
 *
 *    unsigned rep: number of repections for each projection to get a
 *       good running time estimate.
 * 
 * Side effects:
 */
void read_problem(char *restrict problem_name, double aC, char *restrict proj, 
                  unsigned rep);
/*
 * Function Name: free_problem
 *
 * Description: Free memory allocated to hold the SVM problem.
 */
void free_problem();
/*
 * Function: print_statistics 
 *
 * Description: Prints the performance statistics for solving the
 *    problem.
*/
void printStatistics();
/*
 * Function Name: evalf
 *
 * Description: Evaluates the objective functin f(x) = 1/2 x'Kx - e'x.
 *
 * Input:
 *    int n: dimension of x.
 *    double *restrict x: function argument
 *
 * Output:
 *    double *restict f: objective function value
 *    int *restrict flag: Returns 0 to indicate that the objective was
 *       successfully computed.
*/
void evalf(int n, double *restrict x, 
           double *restrict f, int *restrict flag);
/*
 * Function Name: evalg
 * 
 * Description: Compute the gradient of the objective function g(x) =
 *    Kx - e.
 *
 * Input:
 *    int n: dimension of x.
 *    double *restrict x: function argument.
 *
 * Output:
 *    double *restrict g: the gradient
 *    int *restrict flag: Returns 0 to indicate that the objective was
 *       successfully computed.
 *
 * WARNING: This function assumes that evalf was called just before
 *    with the same x.
*/
void evalg(int n, double *restrict x, 
           double *restrict g, int *restrict flag);
/*
 * Function Name: proj
 * 
 * Description: Solve the projection onto 0 <= x <= C, signals'x = 0
 * (the feasible set for the SVMN problem using the correct method.
 *
 * Input:
 *    int n: dimension of x;
 *
 * Input/Outuput: double *restrict x: The point the project at input
 *    and its projection at output.
 *    
 *    int *restrict flag: Returns 0 to indicate that the objective was
 *       successfully computed.
 */
void proj(int n, double *restrict x, int *restrict flag);

#endif
